package com.mysql.cluster.ndbj;

import java.sql.Timestamp;

public interface NdbBaseOperationSet {

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val
	 * @throws NdbApiException if there is a problem in the cluster.
	 */
	public abstract void setShort(long columnId, short val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column the value to be set for the column
	 * @throws NdbApiException
	 */
	public abstract void setShort(String columnName, short val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val
	 * @throws NdbApiException if there is a problem in the cluster.
	 */
	public abstract void setInt(long columnId, int val) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column the value to be set for the column
	 * @throws NdbApiException
	 */
	public abstract void setInt(String columnName, int val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setLong(long columnId, long val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setLong(String columnName, long val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setFloat(long columnId, float val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setFloat(String columnName, float val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setDouble(long columnId, double val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setDouble(String columnName, double val)
			throws NdbApiException;

	/**
	 * Timestamp objects stored in the database lose nanosecond accuracy.
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param t if t is null, then setNull method is called for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setTimestamp(long columnId, Timestamp t)
			throws NdbApiException;

	/**
	 * Timestamp objects stored in the database lose nanosecond accuracy.
	 * @param columnName name of the Column in the Schema
	 * @param t
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column. if columnName is null or empty string
	 * @throws NdbApiTimeStampOutOfBoundsException if Timestamp value is for year
	 * after 2037
	 */
	public abstract void setTimestamp(String columnName, Timestamp t)
			throws NdbApiException, NdbApiTimeStampOutOfBoundsException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setBytes(long columnId, byte[] val)
			throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setBytes(String columnName, byte[] val)
			throws NdbApiException;

	/**
	 * This method first queries the cluster about the charset of columnName,
	 * before setting columnName to val.
	 * The supported charsets are Latin1 and utf8. If columnName has
	 * no charset, it is assume to be VARBINARY, and the charset is assumed to
	 * be latin1.
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setString(long columnId, String val)
			throws NdbApiException;

	/**
	 * This method first queries the cluster about the charsett of columnName,
	 * before setting columnName to val.
	 * The supported charsets are Latin1 and utf8. If columnName has
	 * no charset, it is assume to be VARBINARY, and the charset is assumed to
	 * be latin1.
	 *
	 * @param columnName name of the Column in the Schema
	 * @param val the value to be set for the column
	 * @return
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setString(String columnName, String val)
			throws NdbApiException;

	/**
	 *
	 * @param columnId integer position (offset) of column number in schema definition (columnId starts from  position '1' for the first column in a schema)
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setNull(long columnId) throws NdbApiException;

	/**
	 *
	 * @param columnName name of the Column in the Schema
	 * @throws NdbApiException if there was a problem in the cluster when trying to set a value for a column.
	 */
	public abstract void setNull(String columnName) throws NdbApiException;

}
