package com.mysql.cluster.ndbj;

import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.RollbackException;
import javax.transaction.Synchronization;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.xa.XAResource;

public abstract class NdbJtaTransaction extends NdbOperationFactoryImpl implements Transaction {

	private Synchronization synchronization = null;
	private boolean rollbackOnly = false;
	private boolean inCommit = false;
	
	public abstract int execute(ExecType execType, AbortOption abortOption, boolean force) throws NdbApiException;
	public abstract int execute(ExecType execType, AbortOption abortOption) throws NdbApiException;
	public abstract int execute(ExecType execType) throws NdbApiException;


	public void commit() throws RollbackException, HeuristicMixedException,
			HeuristicRollbackException, SecurityException,
			IllegalStateException, SystemException {
		inCommit = true;
		int retVal = -1;
		if (synchronization != null) 
			synchronization.beforeCompletion();
		try { 
			if (rollbackOnly) { 
				throw new RollbackException("Rollback requested");
			}
			retVal = execute(ExecType.Commit,AbortOption.AbortOnError,false);
		} catch (NdbApiException e) { 
			rollback();
			throw new RollbackException("Transaction failed and was rolled back: "+e.message);
		} catch (RollbackException rbE) { 
			rollback();
			throw rbE;
		} finally { 
			if (synchronization != null)
				synchronization.afterCompletion(retVal);
			inCommit = false;
		}
		
	}

	public boolean delistResource(XAResource arg0, int arg1)
			throws IllegalStateException, SystemException {
		// TODO Auto-generated method stub
		// return false;
		throw new NotImplementedException("delistResource");

	}

	public boolean enlistResource(XAResource arg0) throws RollbackException,
			IllegalStateException, SystemException {
		// TODO Auto-generated method stub
		// return false;
		throw new NotImplementedException("enlistResource");

	}

	public int getStatus() throws SystemException {
		// TODO Auto-generated method stub
		// return 0;
		throw new NotImplementedException("getStatus");

	}

	public void registerSynchronization(Synchronization theSynchronization)
			throws RollbackException, IllegalStateException, SystemException {
			synchronization = theSynchronization;
	}

	public void rollback() throws IllegalStateException, SystemException {
		int retVal = -1;
		if (!inCommit && synchronization != null) {
			synchronization.beforeCompletion();
		}
		try { 
			retVal = execute(ExecType.Rollback,AbortOption.AbortOnError,false);
		} catch (NdbApiException e) { 
			throw new SystemException("Couldn't rollback - something is seriously wrong: " + e.message);
		} finally { 
			if (!inCommit && synchronization != null) {
				synchronization.afterCompletion(retVal);
			}
		}
	}

	public void setRollbackOnly() throws IllegalStateException, SystemException {
		rollbackOnly = true;
	}

}
