package com.mysql.cluster.ndbj;

import java.util.Map;
import java.util.HashMap;

import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbScanOperation.ScanFlag;

public abstract class NdbOperationFactoryImpl implements NdbOperationFactory {

	private NdbOperation.LockMode lockMode = NdbOperation.LockMode.LM_Read;

	private long batch = 0L;

	private long parallel = 0L;

	private final Map<Long, NdbOperationImpl> createdOperations = new HashMap<Long, NdbOperationImpl>();

	protected abstract NdbOperation getNdbOperation(String tableName)
			throws NdbApiException;

	protected abstract NdbOperation getNdbOperation(NdbTable aTable)
			throws NdbApiException;

	protected abstract NdbScanOperation getNdbScanOperation(NdbTable aTable)
			throws NdbApiException;

	protected abstract NdbScanOperation getNdbScanOperation(String tableName)
			throws NdbApiException;

	protected abstract NdbIndexScanOperation getNdbIndexScanOperation(
			String anIndexName, String aTableName) throws NdbApiException;

	protected abstract NdbIndexScanOperation getNdbIndexScanOperation(
			NdbIndex anIndex, NdbTable aTable) throws NdbApiException;

	protected abstract NdbIndexScanOperation getNdbIndexScanOperation(
			NdbIndex anIndex) throws NdbApiException;

	protected abstract NdbIndexOperation getNdbIndexOperation(
			String anIndexName, String aTableName) throws NdbApiException;

	protected abstract NdbIndexOperation getNdbIndexOperation(NdbIndex anIndex,
			NdbTable aTable) throws NdbApiException;

	protected abstract NdbIndexOperation getNdbIndexOperation(NdbIndex anIndex)
			throws NdbApiException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#setOperationLockMode(com.mysql.cluster.ndbj.NdbOperation.LockMode)
	 */
	public void setOperationLockMode(LockMode theLockMode) {
		this.lockMode = theLockMode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#setBatchFlag(long)
	 */
	public void setBatchFlag(long batch) {
		this.batch = batch;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#setParallelFlag(long)
	 */
	public void setParallelFlag(long parallel) {
		this.parallel = parallel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectOperation(java.lang.String,com.mysql.cluster.ndbj.NdbOperation.LockMode)
	 */
	public NdbOperation getSelectOperation(String tableName,
			NdbOperation.LockMode theLockMode) throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(tableName);
		theOperation.readTuple(theLockMode);
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectOperation(java.lang.String)
	 */
	public NdbOperation getSelectOperation(String tableName)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(tableName);
		theOperation.readTuple(lockMode);
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectOperation(com.mysql.cluster.ndbj.NdbTable,com.mysql.cluster.ndbj.NdbOperation.LockMode)
	 */
	public NdbOperation getSelectOperation(NdbTable table,
			NdbOperation.LockMode theLockMode) throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(table);
		theOperation.readTuple(theLockMode);
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectOperation(com.mysql.cluster.ndbj.NdbTable)
	 */
	public NdbOperation getSelectOperation(NdbTable table)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(table);
		theOperation.readTuple(lockMode);
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getUpdateOperation(String tableName)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(tableName);
		theOperation.updateTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getUpdateOperation(NdbTable table)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(table);
		theOperation.updateTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getInsertOperation(String tableName)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(tableName);
		theOperation.insertTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getInsertOperation(NdbTable table)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(table);
		theOperation.insertTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getDeleteOperation(String tableName)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(tableName);
		theOperation.deleteTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbOperation (primary key operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbOperation getDeleteOperation(NdbTable table)
			throws NdbApiException {
		NdbOperationImpl theOperation = (NdbOperationImpl) getNdbOperation(table);
		theOperation.deleteTuple();
		createdOperations.put(new Long(NdbOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	
	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getDeleteUniqueOperation(com.mysql.cluster.ndbj.NdbTable)
	 */
	public NdbIndexOperation getDeleteUniqueOperation(NdbIndex index, NdbTable table)
			throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(index, table);
		theOperation.deleteTuple();
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getDeleteUniqueOperation(java.lang.String)
	 */
	public NdbIndexOperation getDeleteUniqueOperation(String indexName, String tableName)
			throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(indexName, tableName);
		theOperation.deleteTuple();
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getDeleteUniqueOperation(com.mysql.cluster.ndbj.NdbIndex)
	 */
	public NdbIndexOperation getDeleteUniqueOperation(NdbIndex index)
			throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(index);
		theOperation.deleteTuple();
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}
	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectUniqueOperation(com.mysql.cluster.ndbj.NdbIndex, com.mysql.cluster.ndbj.NdbTable, com.mysql.cluster.ndbj.NdbOperation.LockMode)
	 */
	public NdbIndexOperation getSelectUniqueOperation(NdbIndex index,
			NdbTable table, LockMode theLockMode) throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(index,table);
		theOperation.readTuple(theLockMode);
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectUniqueOperation(com.mysql.cluster.ndbj.NdbIndex, com.mysql.cluster.ndbj.NdbTable)
	 */
	public NdbIndexOperation getSelectUniqueOperation(NdbIndex index,
			NdbTable table) throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(index,table);
		theOperation.readTuple(lockMode);
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}


	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectUniqueOperation(java.lang.String, java.lang.String, com.mysql.cluster.ndbj.NdbOperation.LockMode)
	 */
	public NdbIndexOperation getSelectUniqueOperation(String indexName,
			String tableName, LockMode theLockMode) throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(indexName,tableName);
		theOperation.readTuple(theLockMode);
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getSelectUniqueOperation(java.lang.String, java.lang.String)
	 */
	public NdbIndexOperation getSelectUniqueOperation(String indexName,
			String tableName) throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(indexName,tableName);
		theOperation.readTuple(lockMode);
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getUpdateUniqueOperation(com.mysql.cluster.ndbj.NdbTable)
	 */
	public NdbIndexOperation getUpdateUniqueOperation(NdbIndex index, NdbTable table)
			throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(index,table);
		theOperation.updateTuple();
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/* (non-Javadoc)
	 * @see com.mysql.cluster.ndbj.NdbOperationFactory#getUpdateUniqueOperation(java.lang.String)
	 */
	public NdbIndexOperation getUpdateUniqueOperation(String indexName, String tableName)
			throws NdbApiException {
		NdbIndexOperationImpl theOperation = (NdbIndexOperationImpl) getNdbIndexOperation(indexName, tableName);
		theOperation.updateTuple();
		createdOperations.put(new Long(NdbIndexOperationImpl.getCPtr(theOperation)),
				theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName)
			throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(tableName);
		theOperation.readTuples(lockMode);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table)
			throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(table);
		theOperation.readTuples(lockMode);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @param scanFlag
	 *            flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName,
			ScanFlag scanFlag) throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(tableName);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @param scanFlag
	 *            flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table,
			ScanFlag scanFlag) throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(table);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @param scanFlag
	 *            OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(NdbTable table, long scanFlag)
			throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(table);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbScanOperation (table scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param aTable
	 *            the table to be operated upon
	 * @param scanFlag
	 *            OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbScanOperation getSelectScanOperation(String tableName,
			long scanFlag) throws NdbApiException {
		NdbScanOperationImpl theOperation = (NdbScanOperationImpl) getNdbScanOperation(tableName);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param indexName
	 *            the index to be scanned
	 * @param tableName
	 *            the table to be operated upon
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName,
			String tableName) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				indexName, tableName);
		theOperation.readTuples(lockMode);
		createdOperations.put(new Long(NdbIndexScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to
	 * the transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @param table
	 *            the table to be operated upon
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index,
			NdbTable table) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				index, table);
		theOperation.readTuples(lockMode);
		createdOperations.put(new Long(NdbIndexScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index)
			throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(index);
		theOperation.readTuples(lockMode);
		createdOperations.put(new Long(NdbIndexScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param indexName
	 *            the index to be scanned
	 * @param tableName
	 *            the table to be operated upon
	 * @param scanFlag
	 *            flag controlling the operation of the scan
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName,
			String tableName, ScanFlag scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				indexName, tableName);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to
	 * the transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @param table
	 *            the table to be operated upon
	 * @param scanFlag
	 *            flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index,
			NdbTable table, ScanFlag scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				index, table);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbIndexScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @param scanFlag
	 *            flag controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index,
			ScanFlag scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(index);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param indexName
	 *            the index to be scanned
	 * @param tableName
	 *            the table to be operated upon
	 * @param scanFlag
	 *            OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbIndexScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(String indexName,
			String tableName, long scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				indexName, tableName);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates an NdbIndexScanOperation (index scan operation) and adds it to
	 * the transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @param table
	 *            the table to be operated upon
	 * @param scanFlag
	 *            OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index,
			NdbTable table, long scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(
				index, table);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbIndexScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	/**
	 * Creates a NdbIndexScanOperation (Index scan operation) and adds it to the
	 * transaction.
	 * 
	 * @param index
	 *            the index to be scanned
	 * @param scanFlag
	 *            OR'd set of ScanFlags controlling the operation of the scan
	 * @return an NdbScanOperation object
	 * @throws NdbApiException
	 *             if tableName is null or empty, the object has been closed, or
	 *             the Operation object could not be created
	 */
	public NdbIndexScanOperation getSelectIndexScanOperation(NdbIndex index,
			long scanFlag) throws NdbApiException {
		NdbIndexScanOperationImpl theOperation = (NdbIndexScanOperationImpl) getNdbIndexScanOperation(index);
		theOperation.readTuples(lockMode, scanFlag, parallel, batch);
		createdOperations.put(new Long(NdbScanOperationImpl
				.getCPtr(theOperation)), theOperation);
		return theOperation;
	}

	
	
}
