package com.mysql.cluster.ndbj;

import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.TransactionManager;
import javax.transaction.Transaction;

public abstract class NdbTransactionManager implements TransactionManager {

	private final ThreadLocal<NdbTransaction> context = new ThreadLocal<NdbTransaction>();
	
	public void begin() throws NotSupportedException, SystemException {

		NdbTransaction transaction = context.get();
		if (transaction != null) { 
			throw new IllegalStateException("Can't begin - already have a transaction");
		}
		try {
			transaction = startTransaction();
		} catch (NdbApiException e) { 
			throw new SystemException(e.message);
		}
		context.set(transaction);

	}

	public void commit() throws RollbackException, HeuristicMixedException,
			HeuristicRollbackException, SecurityException,
			IllegalStateException, SystemException {
		NdbTransaction transaction = getTransaction();
		transaction.commit();
	}


	public int getStatus() throws SystemException {

		NdbTransaction transaction = getTransaction();
		return transaction.getStatus();
		
	}

	public NdbTransaction getTransaction() throws SystemException {
		NdbTransaction transaction = context.get();
		if (transaction==null) { 
			throw new IllegalStateException("no transaction to commit");
		}
		return transaction;
	}

	public void resume(Transaction arg0) throws InvalidTransactionException,
			IllegalStateException, SystemException {
		NdbTransaction transaction = context.get();
		if (transaction != null) {
			throw new IllegalStateException("Can't resume - already have a transaction");
		} else if (arg0.getClass().isInstance(NdbTransaction.class)) { 
			context.set((NdbTransaction)arg0);
		} else { 
			throw new InvalidTransactionException("Ndb only manages NdbTransactions");
		}

	}

	public void rollback() throws IllegalStateException, SecurityException,
			SystemException {
		NdbTransaction transaction = getTransaction();
		transaction.rollback();
	}

	public void setRollbackOnly() throws IllegalStateException, SystemException {
		// TODO Auto-generated method stub
		// 
		throw new NotImplementedException("setRollbackOnly");
	}

	public void setTransactionTimeout(int arg0) throws SystemException {
		// TODO Auto-generated method stub
		// 
		throw new NotImplementedException("setTransactionTimeout");
	}

	public NdbTransaction suspend() throws SystemException {
		NdbTransaction transaction = getTransaction();
		context.remove();
		return transaction;
	}

	public abstract NdbTransaction startTransaction() throws NdbApiException;

}
