// -*- mode: c++ -*- 
/*  ndb-bindings: Bindings for the NDB API
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


%module(directors="1") mgmj

/*
%typemap(javaimports) NdbMgmFactory, ndb_mgm_handle, ndb_mgm_cluster_state, ndb_mgm_node_state "import com.mysql.cluster.errors.*;"
%pragma(java) moduleimports="import com.mysql.cluster.errors.*;"
%pragma(java) jniclassimports="import com.mysql.cluster.errors.*;"
*/

%typemap(javabody) SWIGTYPE %{
  private long swigCPtr;
  protected boolean swigCMemOwn;

  public $javaclassname(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  public static long getCPtr($javaclassname obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }
%}

%typemap(javabody_derived) SWIGTYPE %{
  private long swigCPtr;

  public $javaclassname(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  public static long getCPtr($javaclassname obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }
%}


%{

#define NDB_exception(excp, msg) { ndb_throw_exception(jenv, #excp, msg);  } 

void ndb_throw_exception(JNIEnv *jenv, const char * excp, const char *msg) { 
  const char * prefix = "com/mysql/cluster/mgmj/";
  char * exception = (char *)malloc(strlen(excp)+strlen(prefix)+1);
  strcpy(exception,prefix);
  strcat(exception,excp);

  //jclass clazz = jenv->FindClass("java/lang/RuntimeException");
  jclass clazz = jenv->FindClass(exception);
  jenv->ThrowNew(clazz,msg);
}
%}

%typemap(out) (BackupWrapper *) { 
  $result=$1->backupId;
  free($1->theReply);
  free($1);
 }
%typemap(jstype) (BackupWrapper *) "long";
%typemap(jni) (BackupWrapper *) "unsigned int";

%typemap(javaout) (BackupWrapper *) {
  return $jnicall;
 }

%typemap(out) (BaseEventWrapper *) {
  $result=$1->theEvent; 
 } 
//%typemap(jtype) (BaseEventWrapper *) "BaseEvent";
%typemap(jstype) (BaseEventWrapper *) "NdbLogEvent";
%typemap(jni) (BaseEventWrapper *) "ndb_logevent *";

%typemap(javaout) (BaseEventWrapper *) { 
  long cPtr = $jnicall; 
  return (cPtr == 0) ? null :  new NdbLogEvent(cPtr,true);
 }


%typemap(javaimports) ndb_mgm_cluster_state %{
import java.util.ArrayList;
import java.util.Collection;
%}

%typemap(javacode) ndb_mgm_cluster_state  %{
  public Collection<NodeState> getNodeStates() {
    Collection<NodeState> nodeStates = new ArrayList<NodeState>(0);
    for (int i=0; i < getNoOfNodes(); i++) {
      nodeStates.add(getNodeState(i));
    }
    return nodeStates;
  }
  %}


/* int * typemap */

/* These 3 typemaps tell SWIG what JNI and Java types to use */
%typemap(jni) (int no_of_nodes, const int * node_list) "jintArray"
%typemap(jtype) (int no_of_nodes, const int * node_list) "int[]"
%typemap(jstype)(int no_of_nodes, const int * node_list) "int[]"

/* This tells SWIG to treat int * as a special case when used as a parameter
   in a function call */
%typemap(in) (int no_of_nodes, const int * node_list) {
    $1 = jenv->GetArrayLength($input);
    jboolean mode = false;
    $2= jenv->GetIntArrayElements($input,&mode);
    /*int* jarr= jenv->GetIntArrayElements($input,&mode);
    $2 = (int *)malloc($1*sizeof(int)+1);
    for (int i=0;i<$1;i++){
      //$2[i]=(int)jarr[i];
    } */
 }

/* This cleans up the memory we malloc'd before the function call */
%typemap(freearg) (int no_of_nodes, const int * node_list)  {
  jenv->ReleaseIntArrayElements($input,$2,0);
  //free($2);
 }

%typemap(argout) (int no_of_nodes, const int * node_list) {

 }

%include "mgmapi/mgmglobals.i"
%include "mgmapi/NdbLogEvent.i"
%include "mgmapi/NdbLogEventManager.i"
%include "mgmapi/NdbMgm.i"
%typemap(javacode) NdbMgmFactory %{ 

  static {
    try {
      System.loadLibrary("mgmj");
    } catch (UnsatisfiedLinkError e) {
      System.err.println("Native code library failed to load. \n" + e);
      System.exit(1);
    }
    mgmj.ndb_init();
  }

 %}
%include "mgmapi/NdbMgmFactory.i"
%include "mgmapi/ClusterState.i"
%include "mgmapi/NodeState.i"
%include "mgmapi/NdbMgmReply.i"
