package testsuite;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbClusterConnection;
import com.mysql.cluster.ndbj.NdbDictionary;
import com.mysql.cluster.ndbj.NdbFactory;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.ndbj;


public class BaseNdbjTestCase extends testsuite.BaseTestCase {



	protected static final String CONFIG_FILENAME="ndbj.props";
	protected static String MYSQLD_HOST="";
	protected static String NDB_MGMD_CONNECTSTRING="";
	protected static String DB_NAME="test";
	protected static String TABLE_NAME="";
	protected static String LIB_PATH = "";
	protected static String USERNAME  = "root";
	protected static String PASSWORD = "";
	protected static String CLUSTER_LOG="";
	protected static Connection MYSQL_CONN=null;
	
	protected NdbClusterConnection ndbConn=null;
	protected com.mysql.cluster.ndbj.Ndb ndb=null;
	protected NdbTransaction trans=null;
	
	static { 
  	    System.loadLibrary("ndbj");
    	ndbj.ndb_init();
	}

	public BaseNdbjTestCase(String arg0) {

		super(arg0);
		/**
		 * Set environment variables
		 */

		File f =null;
		try {
			InputStream in=null;
			f = new File(CONFIG_FILENAME);
			if (f.exists()){
				if (f.canRead()){
					in = new FileInputStream(f);
					}
				else {
					System.out.println("Could not open " + CONFIG_FILENAME);
					assertFalse(true);
					}
				}
			else {
				System.out.println("Could not find " + CONFIG_FILENAME);
				assertFalse(true);
				}

		    Properties p = new Properties(System.getProperties());
		    p.load(in);

			// set the system properties
	        System.setProperties(p);

	        BaseNdbjTestCase.MYSQLD_HOST = System.getProperty("ndbj.mysqld");
			if (BaseNdbjTestCase.MYSQLD_HOST == null) {
				System.out.println("ndbj.mysql not defined in configuration file: + " + CONFIG_FILENAME);
				assertFalse(true);
			}

			BaseNdbjTestCase.NDB_MGMD_CONNECTSTRING= System.getProperty("ndbj.connectstring");
			if (BaseNdbjTestCase.NDB_MGMD_CONNECTSTRING == null) {
				System.out.println("ndbj.connectstring not defined in configuration file: + " + CONFIG_FILENAME);
				assertFalse(true);
			}

			BaseNdbjTestCase.DB_NAME= System.getProperty("ndbj.testDatabase");
			if (BaseNdbjTestCase.DB_NAME == null) {
				System.out.println("ndbj.testDatabase was empty. Setting it to 'test' db.");
				BaseNdbjTestCase.DB_NAME = "test";
			}

			BaseNdbjTestCase.LIB_PATH= System.getProperty("ndbj.libpath");
			if (BaseNdbjTestCase.LIB_PATH == null) {
			}

			BaseNdbjTestCase.USERNAME = System.getProperty("ndbj.username");
			if (BaseNdbjTestCase.USERNAME == null) {
				System.out.println("ndbj.username was empty. Setting it to 'root'.");
			}

			BaseNdbjTestCase.PASSWORD = System.getProperty("ndbj.password");
			if (BaseNdbjTestCase.PASSWORD == null) {
				System.out.println("ndbj.password was empty. Setting it to empty string ''");
			}

			BaseNdbjTestCase.CLUSTER_LOG= System.getProperty("ndbj.clusterLog");
			if (BaseNdbjTestCase.CLUSTER_LOG== null) {
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}


/*
		Class clazz = ClassLoader.class;
        Field field;
        boolean accessible;
        try {
        // Reset the "sys_paths" field of the ClassLoader to null.
        field = clazz.getDeclaredField("sys_paths");
        accessible = field.isAccessible();
        if (!accessible)
            field.setAccessible(true);
        // Reset it to null so that whenever "System.loadLibrary" is called, it will be reconstructed with the changed value.
        field.set(clazz, null);
        System.setProperty("java.library.path", LIB_PATH);
        }
        catch (java.lang.UnsatisfiedLinkError e) {
            System.err.println(e.getMessage());
        }
        catch (NoSuchFieldException e) {
            System.err.println(e.getMessage());
        }
        catch (IllegalAccessException e) {
            System.err.println(e.getMessage());
        }
*/


	}

	@Override
	public void setUp() throws Exception {
	      /**
	       *  Connect to cluster, done once for each Application
	       */
		super.setUp();
		System.out.println("Running test: "+this.getClass().getName());

        //System.out.print("Connecting to cluster...");
        if (ndbConn==null) {
            ndbConn = NdbFactory.createNdbClusterConnection(NDB_MGMD_CONNECTSTRING);

            ndbConn.connect(5,3,true);
            ndbConn.waitUntilReady(30,0);
        }
          /**
           *  Get a connection to NDB (once / thread)
           */
        ndb = ndbConn.createNdb("test",1024);

        //System.out.println("connected");
	}
	
	@Override
	public void tearDown() throws Exception {


		if (trans != null) {
			trans.close();
			trans=null;
		}

		ndb.close();
		ndb=null;
		ndbConn.close();
		ndbConn = null;
		super.tearDown();
	}
	
	/* (non-Javadoc)
	 * @see testsuite.BaseTestCase#createTable(java.lang.String, java.lang.String)
	 */
	@Override
	protected void createTable(String tableName, String columnsAndOtherStuff) throws SQLException {
		super.createTable(tableName, columnsAndOtherStuff);
		NdbDictionary theDict = ndb.getDictionary();
		theDict.invalidateTable(tableName);
	}
	
	
	/* (non-Javadoc)
	 * Doesn't use invalidateTable. Really only for the Invalid Schema Object test. 
	 */
	protected void recreateTable(String tableName, String columnsAndOtherStuff) throws SQLException {
		super.createTable(tableName, columnsAndOtherStuff);
	}
	

}
