package testsuite.ndbj;

import java.sql.SQLException;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbApiTemporaryException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class InsertRetryTest extends BaseNdbjTestCase {

	protected static final String tablename = "insert_retry_test";
	
	protected static final String col1 = "id";

	protected static final String col2 = "name";

	protected static final String col2Val = "montyµ";

	protected static final int MAX_RETRIES = 3;

	protected static final int NUM_INSERTS = 2;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(InsertRetryTest.class);
	}

	public InsertRetryTest(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception { 
		super.setUp();
		this.createTable(tablename, "(id int, name varchar(56), primary key(id)) engine=ndbcluster default charset=utf8");
		
	}

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */
	public void testInsert() throws java.sql.SQLException {

		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction();
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getInsertOperation(tablename);
					op.equalInt(col1, i);
					String setting = col2Val + Integer.toString(i);
					op.setString(col2, setting);
				}
				
				trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				assertTrue("Temporary error. Retrying insert. Num Retries == "
						+ numRetries + " error msg:" + e.getMessage(),true);
			} catch (NdbApiException e) {
				assertTrue("Caught NdbApiException in Insert:\t" + e.getMessage(), false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}
		
		for (int i = 0; i < NUM_INSERTS; i++) { 
			String nameVal = (String)this.getSingleValue(tablename, "name", "where id=" + Integer.toString(i));
			assertEquals(nameVal,col2Val + Integer.toString(i));
		}
	}
	
	public void jdbcInsert() throws SQLException { 
	
		String setting = "";
		
		for (int i = 0; i < NUM_INSERTS ; i++) { 
			setting = col2Val + Integer.toString(i);
			this.stmt.execute("insert into " + tablename + "(id, name) values (" + ((Integer)i).toString()
					+ ", '" + setting + "')");
		}
	}
	public void testRead() throws SQLException {

		this.jdbcInsert();

		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			
			try {

				trans = ndb.startTransaction();

				NdbResultSet[] theResult = new NdbResultSet[NUM_INSERTS];
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getSelectOperation(tablename,
							LockMode.LM_Read);

					theResult[i] = op.resultData();
					op.equalInt(col1, i);
					op.getValue(col2);
				}

				trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);

				int j = 0;
				for (NdbResultSet r : theResult) {
					while (r.next()) {
						String val = r.getString(col2);
						assertEquals(val, col2Val + Integer.toString(j));
					}
					j++;
				}

				assertTrue(true);
				done = true;
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				assertTrue("Temporary error. Retrying Read. Num Retries == "
						+ numRetries + " error msg:" + e.getMessage(), true);
			} catch (NdbApiException e) {
				assertTrue(
						"Caught NdbApiException in Read:\t" + e.getMessage(),
						false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}

	}

	public void testDelete() throws java.sql.SQLException {
		
		this.jdbcInsert();
		
		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {

			try {
				trans = ndb.startTransaction();

				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getDeleteOperation(tablename);
					op.equalInt(col1, i);
				}

				trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);

				assertTrue(true);
				done = true;
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				assertTrue("Temporary error. Retrying delete. Num Retries == "
						+ numRetries + " error msg:" + e.getMessage(), true);
			} catch (NdbApiException e) {
				assertTrue(
						"Caught NdbApiException in delete:\t" + e.getMessage(),
						false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}
		int rowcount = this.getRowCount(tablename);
		assertEquals(rowcount, 0);

	}

}
