package testsuite.ndbj;

import java.sql.SQLException;

import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.InvalidSchemaObjectVersionException;
import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbDictionary;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbTable;

import testsuite.BaseNdbjTestCase;

/**
 * @author mtaylor
 */
public class InvalidSchemaObjectVersionTest extends BaseNdbjTestCase {
	String table_name = "basic_test";
	
	public InvalidSchemaObjectVersionTest(String arg0) { 
		super(arg0);
	}

	/* (non-Javadoc)
	 * @see testsuite.BaseNdbjTestCase#setUp()
	 */
	@Override
	public void setUp() throws Exception {
		super.setUp();
		this.createTable(table_name, "(id int, name varchar(32), age int, primary key(id)) engine=ndbcluster");
	}

	public void testInvalidSchema() throws NdbApiException, SQLException, Exception {
		NdbDictionary dictionary = ndb.getDictionary();
		
		@SuppressWarnings("unused")
		NdbTable table = dictionary.getTable(table_name);

		/* Do it manually to avoid the auto-invalidation done by the test
		 * framework
		 */
		stmt.executeUpdate("drop table " + table_name);
		stmt.executeUpdate("create table " + table_name +
				"(id int, name varchar(32), age int, primary key(id)) engine=ndbcluster");
		
		trans = ndb.startTransaction();
		NdbOperation op = trans.getInsertOperation(table_name);
		op.equalInt("id", 1);
		op.setString("name", "Monty");
		
		/* Now - we _should get an exception here */
		try { 
			trans.execute(ExecType.Commit,AbortOption.AbortOnError,true);
			fail("InvalidSchemaObjectVersionException should be thrown");
		} catch (InvalidSchemaObjectVersionException e) { 
			dictionary.invalidateTable(table_name);
		} catch (Exception e) { 
			e.printStackTrace();
			fail("expected InvalidSchemaObjectVersionException");
		}
		
		trans = ndb.startTransaction();
		op = trans.getInsertOperation(table_name);
		op.equalInt("id", 1);
		op.setString("name", "Monty");
		trans.execute(ExecType.Commit,AbortOption.AbortOnError,true);
	}
}
