package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

/**
 *
create table t_multipart_primary (a int, b int, c varbinary(20), PRIMARY KEY(a,b)) engine=ndb;
insert into t_multipart_primary values(1,11,"jim.......");

 * @author jdowling
 *
 */
public class NdbIndexScanOperationUtf8Latin1Test extends BaseNdbjTestCase {

	protected static final String tablename = "t_multipart_primary";
	private static final String col1Val="jd";
	private static final String col2Val="jp";
	private static final String col3Val="jim";

	protected String col1="a";
	protected String col2="b";
	protected String col3="c";

	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbIndexScanOperationUtf8Latin1Test.class);
	}

	public NdbIndexScanOperationUtf8Latin1Test(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename, "(`a` VARCHAR(256) CHARSET utf8 NOT NULL, "
				+ "`b` VARCHAR(256) CHARSET utf8 NOT NULL, "
				+ "`c` VARCHAR(256), PRIMARY KEY(a,b), KEY (c)) ENGINE=ndb;");
	}



	public void testIndexScanOperationUtf8Latin1() {
	

		try {
			System.out.println("Starting a transaction for table " + tablename);
		    trans = ndb.startTransaction();

		    for (int i=0; i < 5; i++) {
				NdbOperation op = trans.getInsertOperation(tablename);

				if (i > 0) {
					String pk1 = col1Val + i;
					op.equalString(col1,pk1);
					String pk2 = col2Val + i;
					op.equalString(col2,pk2);
				}
				else {
					op.equalString(col1,col1Val);
					op.equalString(col2,col2Val);
				}
				op.setString(col3,col3Val);

		    }
		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Inserted values(" + col1Val + ", " +
					col2Val + ", "
					+ col3Val + ");");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}
	
		try {
			System.out.println("Starting a setbound-eq index scan");
		    trans = ndb.startTransaction();

		    trans.setOperationLockMode(LockMode.LM_Exclusive);
		    
			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getSelectIndexScanOperation("PRIMARY",tablename);

			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);
			op.getValue(col3);

		    NdbResultSet theResult = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(theResult.next())
			{
			    String c = theResult.getString(col3);
			    System.out.println("Read latin1: c=" + c);
			}
		    op.close();
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-eq index scan");
		    trans = ndb.startTransaction();
		    
		    trans.setOperationLockMode(LockMode.LM_Exclusive);
		    
			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getSelectIndexScanOperation("c",tablename);

			op.equalString(col3,col3Val);
			op.getValue(col1);
			op.getValue(col2);

		    NdbResultSet theResult = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(theResult.next())
			{
			    String a = theResult.getString(col1);
			    String b = theResult.getString(col2);
			    System.out.println("Read latin1: a=" + a + ", b=" + b);
			}
		    op.close();

			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-gt index scan");
		    trans = ndb.startTransaction();

		    trans.setOperationLockMode(LockMode.LM_Exclusive);
		    
			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getSelectIndexScanOperation("PRIMARY",tablename);

			op.setBoundString(col1,NdbIndexScanOperation.BoundType.BoundGE,col1Val);
			op.setBoundString(col2,NdbIndexScanOperation.BoundType.BoundGE,col2Val);
			op.getValue(col3);

		    NdbResultSet theResult = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(theResult.next())
			{
			    String c = theResult.getString(col3);
			    System.out.println("Read latin1: c=" + c);
			}
		    op.close();

			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
		    trans = ndb.startTransaction();
		    
		    trans.setOperationLockMode(LockMode.LM_Exclusive);

            NdbIndexScanOperation op =
                trans.getSelectIndexScanOperation("PRIMARY",tablename);
            
			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);
            
            NdbResultSet theResult = op.resultData();

		    trans.execute(ExecType.NoCommit,
			      AbortOption.AbortOnError,
			      true);

            while (theResult.next(true)) {
            	do {
                NdbOperation pkOp = theResult.getUpdateOperation();
                pkOp.setString(col3, col3Val + "_jd");
                trans.execute(ExecType.Commit,
                        AbortOption.AbortOnError,
                        true);
            	} while (theResult.next(false));
            }


            System.out.println("Finished update");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in update:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null) {
				trans.close();
			}
		}

		try {
		    trans = ndb.startTransaction();
		    
		    trans.setOperationLockMode(LockMode.LM_Exclusive);

            NdbIndexScanOperation op =
                trans.getSelectIndexScanOperation("PRIMARY",tablename);
            
			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);

            NdbResultSet theResult = op.resultData();

            trans.execute(ExecType.NoCommit,
                      AbortOption.AbortOnError,
                      true);

            while (theResult.next()) {
                theResult.deleteRow();
            }
            trans.execute(ExecType.Commit,
                      AbortOption.AbortOnError,
                      true);

			System.out.println("Finished delete");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	}

}
