package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

import java.sql.SQLException;

public class NdbOperationByteLongTest extends BaseNdbjTestCase {

	protected final static String tablename = "t_bytes";
	protected String col1="id";
	protected String col2="name";
	protected String col3="num";
	byte[] name = null;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbOperationByteLongTest.class);
	}


	public NdbOperationByteLongTest(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
				"(id VARBINARY(256) PRIMARY KEY, name VARBINARY(256),"
				+ "num BIGINT) ENGINE=ndb;");
	}

	/*
	 * Test method for 'com.mysql.cluster.ndbj.NdbOperation.equalBytes(String, byte[])'
	 */
	public void testInsertAndReadBytesAndLong() {


		try {
			trans = ndb.startTransaction();

			for (int j = 0; j < 10; j++) {
				NdbOperation op = trans.getInsertOperation(tablename);

				byte[] pkID = new byte[5];
				pkID[0] = (byte) ((j >> 24) & 0x000000FF);
				pkID[1] = (byte) ((j >> 16) & 0x0000FF);
				pkID[2] = (byte) ((j >> 8) & 0x00FF);
				pkID[3] = (byte) (j & 0xFF);
				pkID[4] = (byte) -22; // 0xFF

				op.equalBytes(col1, pkID);
				String input = "jim".concat(Integer.toString(j));
				name = input.getBytes();
				op.setBytes(col2, name);
				op.setLong(col3, (long) j * 65536);

			}

			trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);
			assertTrue(true);
		} catch (NdbApiException e) {
			System.out.println("NdbApiException:\t" + e.getMessage());
			assertFalse(true);
		} finally {
			if (trans != null)
				trans.close();
		}


		try {
		    trans = ndb.startTransaction();
		    
			NdbOperation op =trans.getSelectOperation(tablename,LockMode.LM_Exclusive);

			byte[] pkID = new byte[5];
			pkID[0] = (byte)((1 >> 24) & 0x000000FF);
			pkID[1] = (byte)((1 >> 16) & 0x0000FF);
			pkID[2] = (byte)((1 >> 8)  & 0x00FF);
			pkID[3] = (byte)(1 &  0xFF  );
			pkID[4] = (byte) -22; //0xFF

		    op.equalBytes(col1,pkID);
			op.getValue(col1);
			op.getValue(col2);
			op.getValue(col3);

		    rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
			    rs.getBytes(col1);
			    byte[] theName = rs.getBytes(col2);
			    long longVal = rs.getLong(col3);
				System.out.println("\n"
						+ "\nInserted: " + pkID[0]+ ", "
						+ pkID[1]+ ", " + pkID[2]+ ", " + pkID[3]+ ", "
						+ pkID[4]+ ", "
						+ "\nnum=" + longVal);

			    System.out.println("Bytes:");
			    for (int i=0;i< theName.length; i++) {
			    	System.out.print(theName[i]);
			    	System.out.print(", ");
			    }

			}
		}
		catch (SQLException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}

		assertTrue(true);

		System.out.println("Starting an index scan for table " + tablename);

		try {
		    trans = ndb.startTransaction();
		    trans.setOperationLockMode(LockMode.LM_Exclusive);
		    NdbIndexScanOperation op =
		    	trans.getSelectIndexScanOperation("PRIMARY",tablename);

		    byte[] pkID = new byte[5];
			pkID[0] = (byte)((1 >> 24) & 0x000000FF);
			pkID[1] = (byte)((1 >> 16) & 0x0000FF);
			pkID[2] = (byte)((1 >> 8)  & 0x00FF);
			pkID[3] = (byte)(1 &  0xFF  );
			pkID[4] = (byte) -22; //0xFF

			op.equalBytes(col1,pkID);
//		    op.setBound(col1,NdbIndexScanOperationRef.BoundType.BoundEQ, pkID.toString());

			op.getValue(col1);
			op.getValue(col2);
			op.getValue(col3);

		    rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

		    int k=1;
			while(rs.next())
			{
			    rs.getBytes(col1);
			    byte[] theName = rs.getBytes(col2);
			    long longVal = rs.getLong(col3);
//				System.out.println("\n"
//						+ "\nInserted: " + pkID[0]+ ", "
//						+ pkID[1]+ ", " + pkID[2]+ ", " + pkID[3]+ ", "
//						+ pkID[4]+ ", "
//						+ "\nnum=" + longVal);

//			    System.out.println("Bytes:");
//			    for (int i=0;i< name.length; i++) {
//			    	System.out.print(name[i]);
//			    	System.out.print(", ");
//			    }
				String input = "jim".concat(Integer.toString(k));
				byte[] inputBytes = input.getBytes();
				int j=0;
				for (byte b: inputBytes) {
					assertEquals(b,theName[j++]);
				}
				assertEquals(longVal,(long) k*65536);
				k++;
			}
			assertTrue(true);
		}
		catch (SQLException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}

		assertTrue(true);

	}

}
