package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

import java.sql.SQLException;

/**
 *
create table t_multipart_primary (a int, b int, c varbinary(20), PRIMARY KEY(a,b)) engine=ndb;
insert into t_multipart_primary values(1,11,"jim.......");

 * @author jdowling
 *
 */
public class NdbOperationMultiPartPrimaryKeyTest extends BaseNdbjTestCase {

	private static final String col1Val="j";
	private static final String col2Val="d";
	private static final String col3Val="jim";

	protected String col1="a";
	protected String col2="b";
	protected String col3="c";

	protected final static String tablename = "t_str_b";
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbOperationMultiPartPrimaryKeyTest.class);
	}

	public NdbOperationMultiPartPrimaryKeyTest(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception {
		super.setUp();

		createTable("t_str_b", "(`" + col1
				+ "` varchar(256) NOT NULL, "
				+"`" + col2
				+"` varbinary(256), `"
				+ col3
				+ "` varbinary(100), PRIMARY KEY(a,b)) ENGINE=ndb;");
	}


	public void testOperationMultiPartPrimaryKey() {


		try {
			System.out.println("Starting a transaction for table " + tablename);
		    trans = ndb.startTransaction();

			NdbOperation op = trans.getInsertOperation(tablename);

			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);

			op.setBytes(col3,col3Val.getBytes());

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Inserted values(" + col1Val + ", "
					+ col3Val + ");");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting an read transaction");
		    trans = ndb.startTransaction();
		    trans.setOperationLockMode(LockMode.LM_Exclusive);
			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getSelectOperation(tablename);

			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);
			op.getValue(col3);

		    rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
			    byte[] c = rs.getBytes(col3);
			    System.out.println("Read values: c=");
			    for (int i=0; i < c.length; i++)
			    	System.out.print(Byte.toString(c[i]));
			    System.out.println("");
			}
			assertTrue(true);
		}
		catch (SQLException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
		    trans = ndb.startTransaction();

			NdbOperation op = trans.getUpdateOperation(tablename);

			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);
			op.setBytes(col3, (col3Val + "_jd").getBytes());

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished update");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in update:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting an delete transaction");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getDeleteOperation(tablename);

			op.equalString(col1,col1Val);
			op.equalString(col2,col2Val);

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished delete");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	}

}
