package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class NdbOperationsFailedTest extends BaseNdbjTestCase {

	protected static final String tablename = "t_ops";
	
	private static final int NUM_INSERTS=10;
	private int id = 0;

	private String name = "jd";
	protected String col1 = "id";
	protected String col2 = "name";


	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbOperationsFailedTest.class);
	}

	public NdbOperationsFailedTest(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
				"(`id` INT PRIMARY KEY, `name` VARCHAR(256)) ENGINE=ndb;");
	}


	public void testCountNumberOfCompletedOperations() {

		System.out.println("Starting a transaction for table " + tablename);
	

		try {
			trans = ndb.startTransaction();

			for (int j = 0; j < NUM_INSERTS; j++) {
				NdbOperation op = trans.getInsertOperation(tablename);
				
				op.equalInt(col1, id++);
				name = name.concat(Integer.toString(id));
				op.setString(col2, name);
				name = name.substring(0, name.length() - 1);
			}

			// Duplicate insert is also counted as a completed operation (even though it fails!)
			NdbOperation op = trans.getInsertOperation(tablename);
			op.equalInt(col1, id - 1);
			op.setString(col2, name);

			NdbOperation op2 = trans.getInsertOperation(tablename);
			op2.equalInt(col1, id - 1);
			op2.setString(col2, "jim ");

			trans.execute(ExecType.NoCommit, AbortOption.AbortOnError, true);

			// shouldn't get this far
			assertFalse(true);

		} catch (NdbApiException e) {
			System.out.println("Failed:\t" + e.getMessage());

			try {
				int opCount = 0;
				NdbOperation completedOp = trans.getNextCompletedOperation(null);
				do {
					++opCount;
				
				} while ((completedOp = trans.getNextCompletedOperation((com.mysql.cluster.ndbj.NdbOperationImpl)completedOp)) != null);

				assertTrue((opCount==NUM_INSERTS+2));
				System.out.println("Number of completed operations was: " + opCount);
			}
			catch (NdbApiException err) {
				System.out.println("Error while checking for failed operations:\t"
								+ err.getMessage());
				err.printStackTrace();
				assertFalse(true);
			}
		} finally {
			if (trans != null)
				trans.close();
		}



		try {
			trans = ndb.startTransaction();

			for (int j = 0; j < NUM_INSERTS; j++) {
				NdbOperation op = trans.getInsertOperation(tablename);
				op.equalInt(col1, id++);
				name = name.concat(Integer.toString(id));
				op.setString(col2, name);
				name = name.substring(0, name.length() - 1);
			}

			NdbOperation op = trans.getInsertOperation(tablename);
			op.equalInt(col1, id - 1);
			op.setString(col2, name);

			// This operation should fail - duplicate insert
			NdbOperation op2 = trans.getInsertOperation(tablename);
			op2.equalInt(col1, id - 1);
			op2.setString(col2, "jim ");

			trans.execute(ExecType.NoCommit,
					AbortOption.AbortOnError, true);

			// shouldn't get this far
			assertFalse(true);

		} catch (NdbApiException e) {
			System.out.println("Failed:\t" + e.getMessage());
			// TODO: getNextFailedOperation()
			assertTrue("Commented out section with getNextFailedOperation - doesn't exist yet",1==1);
			/*
			try {
				int failedOps=0;
				NdbOperation failedOp = trans.getNextFailedOperation(null);
				if (failedOp != null) {
					do {
						failedOps++;
						System.out.println("Failed operation error was:\t"
								+ failedOp.getNdbError().getMessage());
					} while ((failedOp = trans.getNextFailedOperation(failedOp)) != null);
				}
				assertTrue((failedOps==NUM_FAILED_INSERTS));
			}
			catch (NdbApiException err) {
				System.out.println("Error while checking for failed operations:\t"
								+ err.getMessage());
				assertFalse(true);
			}*/
		} finally {
			if (trans != null)
				trans.close();
		}

	}

//	public void testSelectMissingTupleGetNextFailedOperation() {
//		System.out.println("Starting a transaction for table " + tablename);
//
//		try {
//			trans = ndb.startTransaction();
//
//			NdbOperationRef op = trans.getNdbOperation(tablename);
//			op.committedRead();
//			op.equal(col1, 1000);
//			op.getValue(col2);
//			NdbResultSet rs = op.resultData();
//
//			trans.execute(NdbTransactionRef.ExecType.NoCommit,
//					NdbTransactionRef.AbortOption.AbortOnError, 1);
//
//			// shouldn't get this far
//			assertFalse(true);
//
//		}
//		catch (NdbApiException e) {
//			System.out.println("Failed:\t" + e.getMessage());
//
//			try {
//				int opCount = 0;
//
//				NdbOperationRef failedOp = trans.getNextFailedOperation(null);
//				if (failedOp != null) {
//					do {
//						++opCount;
//						System.out.println("Failed operation error was:\t"
//								+ failedOp.getNdbError().getMessage());
//					} while ((failedOp = trans.getNextFailedOperation(failedOp)) != null);
//					assertTrue(opCount == 1);
//				}
//			}
//			catch (NdbApiException err) {
//				System.out.println("Error while checking for failed operations:\t" + err.getMessage());
//				assertFalse(true);
//			}
//		}
//		finally {
//			if (trans != null)
//				trans.close();
//		}
//
//
//
//	}
}
