package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbScanOperation;

import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;

import java.sql.SQLException;

/**
 *
 * Create this table in the 'test' database, in order to run this unit-test:
 *
 * GARAGE | CREATE TABLE `GARAGE` (
 *   `REG_NO` int(10) unsigned NOT NULL DEFAULT '0',
 *     `BRAND` char(20) NOT NULL DEFAULT '',
 *       `COLOR` char(20) NOT NULL DEFAULT ''
 *       ) ENGINE=ndbcluster DEFAULT CHARSET=latin1 PARTITION BY KEY ()
 *
 * LOCK TABLES `GARAGE` WRITE;
 * INSERT INTO `GARAGE` VALUES (0,'Mercedes\0\0\0\0\0\0\0\0\0\0\0\0','Blue\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(13,'Toyota\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Pink\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(5,'BMW\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Black\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(12,'Toyota\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Pink\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(1,'Mercedes\0\0\0\0\0\0\0\0\0\0\0\0','Blue\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(9,'BMW\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Black\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(8,'BMW\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Black\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(2,'Mercedes\0\0\0\0\0\0\0\0\0\0\0\0','Blue\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(3,'Mercedes\0\0\0\0\0\0\0\0\0\0\0\0','Blue\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(10,'Toyota\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Pink\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(11,'Toyota\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Pink\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(4,'Mercedes\0\0\0\0\0\0\0\0\0\0\0\0','Blue\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(14,'Toyota\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Pink\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(7,'BMW\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Black\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0'),(6,'BMW\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0','Black\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0');
 * UNLOCK TABLES;

 *
 */
public class NdbScanTest extends BaseNdbjTestCase {

	protected final static int NUM_ROWS=10;
	protected final static String tablename="t_scan_op";
	protected String col1="REG_NO";
	protected String col2="BRAND";
	protected String col3="COLOR";
	protected String col2Val="bmw";
	protected String col3Val="blue";

	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbScanTest.class);
	}

	public NdbScanTest(String arg0) {
		super(arg0);
	}

	@Override
	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
				 "(`REG_NO` int(10) unsigned NOT NULL DEFAULT '0' PRIMARY KEY,"
				+"`BRAND` char(20) NOT NULL DEFAULT '',"
				+"`COLOR` char(20) NOT NULL DEFAULT ''"
				+") ENGINE=ndb;"
				);
	}

	public final void testScan() {



		try {
			System.out.println("Starting a transaction for table " + tablename);
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);

			for (int i=0;i<NUM_ROWS;i++)
			{
				NdbOperation op = trans.getInsertOperation(tablename);
				
				op.equalInt(col1,i);

				op.setString(col2,col2Val);
				op.setString(col3,col3Val);
			}
			    trans.execute(ExecType.Commit,
				      AbortOption.AbortOnError,
				      true);


			System.out.println("Finished inserting some rows");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			trans = ndb.startTransaction();

			trans.setOperationLockMode(LockMode.LM_CommittedRead);
		    NdbScanOperation sop =
				trans.getSelectScanOperation(tablename);

			    sop.getValue(col2);
			    sop.getValue(col3);
			    
			    rs = sop.resultData();

			    trans.execute(ExecType.Commit,
					      AbortOption.AbortOnError,
					      true);

			    int rowCount=0;
			    while(rs.next())
			    {
			      /*
			       * getString(columnId)
			       */
			      String testVal=rs.getString(col2);
			      assertEquals(testVal, col2Val);
			      rowCount++;
			    }
			    sop.close();
			    assertEquals(rowCount,NUM_ROWS);

		}
		catch (SQLException e) {
			System.err.println("Exception:" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}

	}


}
