import os, base64,string
from NdbObject import NdbObject

from datetime import datetime

from sqlobject import *

#from turbogears import identity, config
#from stagefiles.widgets import *
#from stagefiles import Errors

#from turbogears.i18n.data.en import months



class ContentObject(NdbObject):

    parent = ForeignKey("ContentObject",default=0)#IntCol(default=0)
    name = StringCol(length=30)
    title = StringCol(length=60,default='')
    description = StringCol(length=255,default='')
    position = IntCol(default=1)
    user = ForeignKey("User")
    image_count = IntCol(default=0)
    user_idx = DatabaseIndex(user)

#    widget = ContentObjectWidget()
#    view_widget = ContentObjectViewWidget()
#    edit_widget = ContentObjectEditWidget()
    
    def getContentItems(self):
        """ Return a list of contained ContentObjects """
        return ContentObject.selectBy(parentID=self.id)

    def getParentContentItems(self):
        """ Return a list of the sibling ContentObjects """
        return ContentObject.selectBy(parentID=self.parent.id)

    def raiseImageCount(self):
        """ Increases the image count in the current object and all
        parent objects"""
        self.image_count=self.image_count+1
        self.parent.raiseImageCount()

    def getNamePath(self):
        return "/".join([self.parent.getNamePath(),self.name])


    def getBreadCrumbs(self):
        bc=list(self.parent.getBreadCrumbs())
        bc.append(self)
        return bc

    def getViewableChildren(self):
        return ContentObject.select("parent_id=%s and image_count>0 and child_name!='Image'" % self.id)

    def getImages(self):
        return ContentObject.select("parent_id=%s and child_name='Image'" % self.id)
    
    def getFirstImage(self):
        imgs=list(ContentObject.select("parent_id=%s and image_count>0" % self.id))
            
        log.debug(imgs)
        return imgs[0].getFirstImage()

class Project(ContentObject):

    director = StringCol(length=32,default='')
    producer = StringCol(length=64,default='')
    location = StringCol(length=32,default='')
    venue = StringCol(length=32,default='')
    start = DateCol(default='1900-01-01')
    end = DateCol(default='2399-12-31')

#    widget = ProjectWidget()
#    edit_widget = ProjectEditWidget()

    def getDateRange(self):
        dates=[]
        if self.start>datetime(1900,01,01).date():
            dates.append(self.start)
        if self.end<datetime(2399,12,01).date():
            dates.append(self.end)

        if self.start.year==self.end.year:
            return "%s %s" % ("-".join([self.monthname(f.month) for f in dates]), self.start.year)
        return "-".join(["%s %s" % (self.monthname(f.month),f.year) for f in dates])
    def monthname(self, month):
        return months[int(month)-1]

#    view_widget = ProjectViewWidget()

class Image(ContentObject):
    _inheritable=False

    caption=StringCol(default='')
    width=IntCol(default=0)
    height=IntCol(default=0)
    extension=StringCol(default='',length=12)
    image_files = MultipleJoin("ImageFile")

    def getFirstImage(self):
        return self

    def getImageLocation(self):
        image_root=config.get('tg.imageroot')
        return os.path.join(image_root,self.getNamePath())

    def getImageFSLocation(self):
        image_root=config.get('tg.imagefs')
        return os.path.join(image_root,self.getNamePath())
        
    def getImagePath(self,size='default'):
        x=self.image_files
        return "%s_%s.%s" % (self.getImageLocation(), size, self.extension)

    def getImageFSPath(self,size='default'):
        return "%s_%s.%s" % (self.getImageFSLocation(), size, self.extension)

    def getDefaultImagePath(self):
        return self.getImagePath(size='default')
    
    def _SO_finishCreate(self, id=None):
        self.raiseImageCount()
        SQLObject._SO_finishCreate(self,id)


    def setImage(self, data):

        path = self.getImageFSLocation()

        ext,w,h = identify(data)
        if not ext or not w or not h:
            raise Errors.ImageError
        self.width = w
        self.height = h
        self.extension=ext
        origpath = self.getImageFSPath('original') #'%s_original.%s' % (path, ext)
        base = os.path.dirname(origpath)
        if not os.path.exists(base): os.makedirs(base)
        f = open (origpath, 'w')
        f.write(data)
        f.close()
        for iversion in ImageVersion.select():
            newpath = self.getImageFSPath(iversion.name) #'%s_%s.%s' % (path, params['name'], ext)
            resize(origpath, w, h,
                   newpath, iversion.width, iversion.height,
                   iversion.enlarge)
            newdata_file = open(newpath,'r')
            newdata = newdata_file.read()
            newdata_file.close()
            newext, neww, newh = identify(newdata)
            ImageFile(name=iversion.name,width=neww,heigh=newh,image=self)

#    widget = ImageWidget()
#    view_widget = ImageViewWidget()

class ImageFile(SQLObject):

    name=StringCol(length=24)
    width=IntCol(default=0)
    height=IntCol(default=0)
    image=ForeignKey("Image")

class Portfolio(ContentObject):
    _inheritable=False

    
    def getPortfolioID(self):
        return self.id

class User(ContentObject):
    """
    Reasonably basic User definition. Probably would want additional attributes.
    """

    # names like "Group", "Order" and "User" are reserved words in SQL
    # so we set the name to something safe for SQL
    _inheritable=False
 
    user_name = UnicodeCol(length=16, alternateID=True,
                           alternateMethodName="by_user_name")
    emails = MultipleJoin("Email")
    addresses = MultipleJoin("Address")
    password = UnicodeCol(length=40)
    created = DateTimeCol(default=datetime.now)

    # groups this user belongs to
    groups = RelatedJoin("Group", intermediateTable="user_group",
                         joinColumn="user_id", otherColumn="group_id")

#    widget=UserWidget()
#    view_widget=UserViewWidget()
    
    def _get_permissions(self):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms
        
    def _set_password(self, cleartext_password):
        "Runs cleartext_password through the hash algorithm before saving."
        hash = identity.encrypt_password(cleartext_password)
        self._SO_set_password(hash)
        
    def set_password_raw(self, password):
        "Saves the password as-is to the database."
        self._SO_set_password(password)


    def getParentContentItems(self):
        """ Return a list of the sibling ContentObjects """
        return None

    def getBreadCrumbs(self):
        return [self]


    def getNamePath(self):
        return self.name

    def raiseImageCount(self):
        """ Increases the image count in the current object and all
        parent objects"""
        self.image_count=self.image_count+1



class ContactType(SQLObject):

    name = StringCol(length=40, alternateID=True,
                     alternateMethodName="by_name")
    description = StringCol(default="")
    
class Email(SQLObject):

    user = ForeignKey("User")
    email = StringCol(length=80, alternateID=True,
                      alternateMethodName="by_email")
    contact_type = ForeignKey("ContactType")
    created = DateTimeCol(default=datetime.now)

class Address(SQLObject):

    user = ForeignKey("User")
    line1 = StringCol(length=80)
    line2 = StringCol(length=80)
    city = StringCol(length=80)
    state= StringCol(length=8)
    zipcode = StringCol(length=12)
    country = StringCol(length=4)
    contact_type = ForeignKey("ContactType")
    created = DateTimeCol(default=datetime.now)
    
class Visit(SQLObject):

    visit_key= StringCol( length=40, alternateID=True,
                          alternateMethodName="by_visit_key" )
    created= DateTimeCol( default=datetime.now )
    expiry= DateTimeCol()

    def lookup_visit( cls, visit_key ):
        try:
            return cls.by_visit_key( visit_key )
        except SQLObjectNotFound:
            return None
    lookup_visit= classmethod(lookup_visit)

class VisitIdentity(SQLObject):
    visit_key = StringCol(length=40, alternateID=True,
                          alternateMethodName="by_visit_key")
    user_id = IntCol()


class Group(SQLObject):
    """
    An ultra-simple group definition.
    """
   
    # names like "Group", "Order" and "User" are reserved words in SQL
    # so we set the name to something safe for SQL
    
    group_name = UnicodeCol(length=16, alternateID=True,
                            alternateMethodName="by_group_name")
    display_name = UnicodeCol(length=255)
    created = DateTimeCol(default=datetime.now)

    # collection of all users belonging to this group
    users = RelatedJoin("User", intermediateTable="user_group",
                        joinColumn="group_id", otherColumn="user_id")

    # collection of all permissions for this group
    permissions = RelatedJoin("Permission", joinColumn="group_id", 
                              intermediateTable="group_permission",
                              otherColumn="permission_id")


    
class Permission(SQLObject):
    permission_name = UnicodeCol(length=16, alternateID=True,
                                 alternateMethodName="by_permission_name")
    description = UnicodeCol(length=255)
    
    groups = RelatedJoin("Group",
                        intermediateTable="group_permission",
                         joinColumn="permission_id", 
                         otherColumn="group_id")


class ImageVersion(SQLObject):
    name = UnicodeCol(length=16,alternateID=True,
                      alternateMethodName="by_name")
    width = IntCol()
    height = IntCol()
    enlarge = BoolCol(default=False)
    
def identify(data):
    (p_in, p_out, p_err) = os.popen3("/usr/bin/identify -verbose -")

    ext,w,h=None,None,None

    p_in.write(data)
    p_in.close()
    while 1:
        ln = p_out.readline()
        if ln == '':
            break

        try:
            k,v = map(string.strip, ln.split(':', 1))
        except:
            continue
        if k=='Geometry':
            w,h = map(int, v.split('x'))
        if k=='Format':
            ext = v.split()[0].lower()
    p_out.close()
    p_err.close()
    return ext,w,h

def resize(curpath, curw, curh, newpath, neww, newh, enlarge):
    # If we add an image that's too small, we don't want to make it bigger.
    if not neww: neww = curw
    if not newh: newh = curh
    if curw<neww and not enlarge:
        neww=curw
    base = os.path.dirname(newpath)
    if not os.path.exists(base): os.makedirs(base)
    os.system('/usr/bin/convert -scale %dx%d %s %s ' % (
        neww,newh,curpath,newpath))
