// -*- mode: c++ -*-
/*  ndb-bindings: Bindings for the NDB API
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

%module "mgmapi"

%include "mgmapi/mgmglobals.i"

%typemap(in) int filter[] { 
  // Input will be array of tuples
  int size = RARRAY($input)->len;
//  int filter[size*2+1];
  $1 = (int *) malloc((size*2+1)*sizeof(int));
  int i=0;
  for (i=0;i<size;i++) { 
    VALUE elem = rb_ary_entry($input,i);
    int elemsize = RARRAY(elem)->len;
    for (int j=0;j<elemsize;j++) { 
      $1[i++]=NUM2INT(rb_ary_entry(elem,j));
    }
  }
  //filter[i]=0; 
  $1[i]=0;
}

%{ 

#define NDB_exception(code,msg) do { ndb_raise_exception(code, msg); SWIG_fail; } while(0);

#define getExceptionMethod(excptype,eparent) \
  SWIGINTERN VALUE \
get ## excptype () { \
  static int init ## excptype = 0 ; \
  static VALUE rb_e ## excptype ; \
  VALUE rb_eparent = ndb_get_exception (eparent); \
  if (! init ## excptype ) { \
    init ## excptype = 1; \
    rb_e ## excptype = rb_define_class(#excptype , rb_eparent); \
  } \
  return rb_e ## excptype ; \
}
VALUE ndb_get_exception(NdbException excpcode);

getExceptionMethod(NdbApiException,BaseRuntimeError);
getExceptionMethod(NdbApiRuntimeException,NdbApiException);
getExceptionMethod(NdbApiPermanentException,NdbApiException);
getExceptionMethod(NdbApiTemporaryException,NdbApiException);
getExceptionMethod(NdbApiUserErrorPermanentException,NdbApiPermanentException);
getExceptionMethod(NdbClusterConnectionPermanentException,NdbApiPermanentException);
getExceptionMethod(NdbClusterConnectionTemporaryException,NdbApiTemporaryException);
getExceptionMethod(NdbApiTimeStampOutOfBoundsException,NdbApiException);
getExceptionMethod(BlobUndefinedException,NdbApiException);
getExceptionMethod(NoSuchColumnException,NdbApiException);
getExceptionMethod(NoSuchIndexException,NdbApiException);
getExceptionMethod(NoSuchTableException,NdbApiException);

void ndb_raise_exception(NdbException excpcode, const char * msg) {
  rb_raise(ndb_get_exception(excpcode),msg);
}

#define CASE_exception(excp) case excp : exception = get ## excp (); break;

VALUE ndb_get_exception(NdbException excpcode) { 

 VALUE exception;

 switch (excpcode) {
    case BaseRuntimeError:
      exception = rb_eRuntimeError;
      break;
    CASE_exception(NdbApiException)
    CASE_exception(BlobUndefinedException)
    CASE_exception(NdbApiPermanentException)
    CASE_exception(NdbApiRuntimeException)
    CASE_exception(NdbApiTemporaryException)
    CASE_exception(NdbApiTimeStampOutOfBoundsException)
    CASE_exception(NdbApiUserErrorPermanentException)
    CASE_exception(NdbClusterConnectionPermanentException)
    CASE_exception(NdbClusterConnectionTemporaryException)
    CASE_exception(NoSuchColumnException)
    CASE_exception(NoSuchIndexException)
    CASE_exception(NoSuchTableException)
    default:
      exception = rb_eRuntimeError;
      break;
  }
  return exception;
}

%}

//%apply ndb_logevent *OUTPUT { ndb_logevent *dst };

%include "mgmapi/mgmglobals.i"
%include "mgmapi/NdbLogEvent.i"
%include "mgmapi/NdbLogEventManager.i"
%include "mgmapi/NdbMgm.i"

%include "mgmapi/NdbMgmFactory.i"
%include "mgmapi/ClusterState.i"
%include "mgmapi/NodeState.i"
%include "mgmapi/NdbMgmReply.i"

